package com.example.civilizedcity.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.*;
import com.example.civilizedcity.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.example.civilizedcity.service.TaCheckItemService;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 测评点位;(ta_check_item)表服务实现类
 *
 * @author : http://www.chiner.pro
 * @date : 2022-12-13
 */
@Service
public class TaCheckItemServiceImpl extends BaseServiceImpl<TaCheckItemMapper, TaCheckItem> implements TaCheckItemService {

    @Autowired
    TaCheckItemQuMapper taCheckItemQuMapper;

    @Autowired
    TaCheckItemAnMapper taCheckItemAnMapper;

    @Autowired
    TaCheckAnswerItemMapper taCheckAnswerItemMapper;

    @Autowired
    TaCheckAnswerMapper taCheckAnswerMapper;

    @Override
    public IPage<TaCheckItem> getPageBy(IPage<TaCheckItem> pg, String checkId, String itemType) {
        return baseMapper.getPageBy(pg, checkId, itemType);
    }

    @Override
    public void createNewItems(TaCheck taCheck) {
        // 先创建实地测试
        baseMapper.createNewLocItems(taCheck.getCheckId());
        // 再创建调查问卷
        TaCheckItem taCheckItem = new TaCheckItem();
        taCheckItem.setCheckId(taCheck.getCheckId());
        taCheckItem.setItemType(Constants.CHECK_OF_SURVEY);
        taCheckItem.setName("调查问卷");
        save(taCheckItem);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void answer(TaCheckItem taCheckItem, TaCheckAnswer taCheckAnswer) throws Exception {
        List<TaCheckItemQu> quList = taCheckItemQuMapper.getListBy(taCheckItem.getItemId());
        if (null == quList || quList.size() < 1) {
            throw new Exception("校验测评问题列表出错");
        }

        List<TaCheckAnswerItem> answerItemList = taCheckAnswer.getAnswerItemList();
        if (null == answerItemList || answerItemList.size() != quList.size()) {
            throw new Exception("未完成全部测评问题");
        }

        // 答案列表如果没有关联问题ID
        long cnt = answerItemList.stream().filter(f -> null == f.getQuId()).count();
        if (cnt > 0) {
            throw new Exception("验证部分题干信息出错");
        }

        if (Constants.CHECK_OF_LOC.equals(taCheckItem.getItemType())) {
            // 如果是实地问卷, 试卷份数不能多于规定份数
            int count = taCheckAnswerMapper.countByItem(taCheckItem.getItemId());
            if (count >= taCheckItem.getNum()) {
                throw new Exception("当前实地问卷已收齐, 无须作答");
            }
        }

        double[] scores = answerItems(taCheckItem, taCheckAnswer, quList, answerItemList);

        if (Constants.CHECK_OF_SURVEY.equals(taCheckItem.getItemType())) {
            // 调查问卷不需要计算当前试卷的分数, 最后才能计算出来
            taCheckAnswerMapper.insert(taCheckAnswer);

            // 但是需要更新答卷份数
            int count = taCheckAnswerMapper.countByItem(taCheckItem.getItemId());
            taCheckItem.setNum(count);
            updateById(taCheckItem);
            return;
        }

        if (Constants.CHECK_OF_LOC.equals(taCheckItem.getItemType())) {
            // 实地调查, 需要计算当前这份试卷的份数
            double positiveScore = scores[0];
            double negativeScore = scores[1];

            // 每份试卷的得分 = 总分 / 份数
            double maxScore = taCheckItem.getFullScore()  / taCheckItem.getNum();
            double totalScore = maxScore + negativeScore + positiveScore;

            if (0 > totalScore) {
                // 如果扣分项太多
                totalScore = 0.0;
            }
            if (totalScore > maxScore) {
                // 如果得分超过总分
                totalScore = maxScore;
            }

            taCheckAnswer.setScore(totalScore);
            taCheckAnswerMapper.insert(taCheckAnswer);
        }
    }

    /**
     * 计算每份试卷份数
     * @param taCheckItem
     * @param taCheckAnswer
     * @throws Exception
     */
    private double[] answerItems(TaCheckItem taCheckItem, TaCheckAnswer taCheckAnswer, List<TaCheckItemQu> quList, List<TaCheckAnswerItem> answerItemList) throws Exception {

        double negativeScore = 0.0; // 扣分
        double positiveScore = 0.0; // 得分
        for(TaCheckAnswerItem item : answerItemList) {
            // 找到对应的题目
            TaCheckItemQu qu = getQu(quList, item.getQuId());
            // + 代表得分, 否则代表扣分
            int sign = "+".equals(qu.getComputeType()) ? 1 : -1;

            double score = 0.0;
            if (Constants.QU_OF_FILL.equals(qu.getQuType())) {
                // 如果是填空题, 用户填的应该是数字, 没填就是 0
                Double an = StringUtils.isEmpty(item.getAnswer()) ? 0.0 : Double.parseDouble(item.getAnswer());
                score = an * qu.getAnScore();
                if (score > qu.getMaxScore()) {
                    score = qu.getMaxScore();
                }

                score *= sign;
            } else if (Constants.QU_OF_RADIO.equals(qu.getQuType())) {
                // 如果是选择题, 先找到对应的答案
                TaCheckItemAn standAn = taCheckItemAnMapper.getOneBy(qu.getQuId(), item.getAnswerCode());

                score = standAn.getScore();
                if (score > qu.getMaxScore()) {
                    score = qu.getMaxScore();
                }

                score *= sign;
            }

            item.setItemId(StringUtils.uuid());
            item.setAnswerId(taCheckAnswer.getAnswerId());
            item.setScore(score);
            taCheckAnswerItemMapper.insert(item);

            if (score > 0) {
                positiveScore += score;
            } else {
                negativeScore += score;
            }
        }

        return new double[] {positiveScore, negativeScore};
    }

    private TaCheckItemQu getQu(List<TaCheckItemQu> quList, String quId) {
        return quList.stream().filter(q -> q.getQuId().equals(quId)).findFirst().orElse(null);
    }
}